/*
 * Hi-Tech C/Xc8 has internal macros : __delay(cycles), __delay_us(uS), __delay_ms(mS)
 *
 * These are custom delay routines, will only be correct for 16 Mhz, 8Mhz, 4Mhz clock.
 *
 */

#include "delay.h"

#if _XTAL_FREQ == 16000000UL
    
    /* function calls and returns are 4 cycles, so on 16Mhz that's 1us
     * so all function belows actually delay for us-1
    */

    void delay_25us(){
        delay_10us(); delay_10us();
        delay_1us(); delay_1us(); delay_1us(); delay_1us();

    }

    void delay_50us(){
        delay_10us(); delay_10us(); delay_10us(); delay_10us();
        delay_5us(); delay_1us(); delay_1us(); delay_1us(); delay_1us();

    }

    void delay_100us(){
        delay_50us();
        delay_10us(); delay_10us(); delay_10us(); delay_10us();
        delay_5us();delay_1us(); delay_1us(); delay_1us(); delay_1us();
    }

    void delay_1ms(){

        char n = 248;
        do {
        asm("goto $+1"); asm("goto $+1"); asm("goto $+1");
        asm("goto $+1"); asm("goto $+1"); asm("goto $+1");
        asm("nop");
        // decrease if not zero - decfsz (1 cycle if !=0, 2 cycles otherwise)
        // goto loop start = 2 cycles

        } while(--n);

        // = 16 cycles x [n+1] + 1 (from last decfsz) - 2 (last goto is skipped)
        // 249x16+1-2 = 3983 cycles (1 = decfsz ), -2 for last goto
        // + 1        = 3984 cycles (assignment of 248 to n)
        // + 4        = 3988 cycles (function call and return)
        delay_2tcy();   // = 3990 cycles
        delay_10tcy();  // = 4000 cycles
    }


    // Delay up to 255 milliseconds - approximately (ms + 19 cycles).
    void delay_ms(char ms) {
        char n;
        while(ms--) {    // time in cycles for the inner loop including
            n = 248;     // cycles = 19+16*ms*[1+n]

            delay_4tcy();
            do {
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("nop");
                // decrease if not zero - decfsz (1 cycle if !=0, 2 cycles otherwise)
                // goto loop start  - 2 cycles

            } while(--n);
            // = 16 cycles x [n+1] + 1 (from last decfsz)
        }
    }
#endif

#if _XTAL_FREQ == 8000000UL

    /* function calls and returns are 4 cycles, so on 8Mhz that's 2us
     * so all function belows actually delay for us-2
    */

    void delay_25us(){
        delay_10us();delay_10us();
        delay_1us(); delay_1us(); delay_1us();

    }

    void delay_50us(){
        delay_10us(); delay_10us(); delay_10us(); delay_10us();
        delay_5us(); delay_1us(); delay_1us(); delay_1us();

    }

    void delay_100us(){
        delay_50us();
        delay_10us(); delay_10us(); delay_10us(); delay_10us();
        delay_5us();delay_1us(); delay_1us(); delay_1us();
    }

    void delay_1ms(){

        char n = 124;
        do {
        asm("goto $+1"); asm("goto $+1"); asm("goto $+1");
        asm("goto $+1"); asm("goto $+1"); asm("goto $+1");
        asm("nop");
        // decrease if not zero - decfsz (1 cycle if !=0, 2 cycles otherwise)
        // goto loop start = 2 cycles

        } while(--n);

        /* = 16 cycles x [n+1] + 1 (from last decfsz) - 2 (last goto is skipped)
        * 125x16+1-2 = 1999 cycles (1 = decfsz ), -2 for last goto
        * + 1        = 2000 cycles (assignment of 248 to n)
        * + 4        = 2004 cycles (function call and return)
        *
        * so technically it's a 1.0002 ms delay
        */
        
    }


    // Delay up to 255 milliseconds - approximately (ms + some cycles).

    void delay_ms(char ms) {
        char n;
        while(ms--) {    
            n = 124;
            do {
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("nop");
                // +2 cycles :: --n : decrease if not zero - decfsz (1 cycle if !=0, 2 cycles otherwise)
                // +2 cycles :: goto loop start
            } while(--n);
            /* = 16 cycles x [n+1] + 1 (from last decfsz) -2 (last goto) = 16 x 125 + 1 = 1999 cycles
             * + ms * 1 cycle for n assignment
             * + ms * 1 cycle for decfsz + 1 cycle (when it's 0)
             * + ms * 2 cycle for loop
             *
             * ex for 10ms = 1999*10 + 10*1 + 10*1 + 1 + 10*2 = 20031 cycles
             */
        }
    }
#endif

#if _XTAL_FREQ == 4000000UL

    /* function calls and returns are 4 cycles, so on 4Mhz that's 4us
     * so all function belows actually delay for us-4
    */

    void delay_25us(){
        delay_10us();delay_10us();
        delay_1us(); 

    }

    void delay_50us(){
        delay_10us(); delay_10us(); delay_10us(); delay_10us();
        delay_5us(); delay_1us();

    }

    void delay_100us(){
        delay_50us();
        delay_10us(); delay_10us(); delay_10us(); delay_10us();
        delay_5us();delay_1us(); 
    }

    void delay_1ms(){

        char n = 61;
        do {
        asm("goto $+1"); asm("goto $+1"); asm("goto $+1");
        asm("goto $+1"); asm("goto $+1"); asm("goto $+1");
        asm("nop");
        // decrease if not zero - decfsz (1 cycle if !=0, 2 cycles otherwise)
        // goto loop start = 2 cycles

        } while(--n);

        // = 16 cycles x [n+1] + 1 (from last decfsz) - 2 (last goto is skipped)
        // 62x16+1-2 = 992 cycles (1 = decfsz ), -2 for last goto
        // + 1        = 993 cycles (assignment of 248 to n)
        // + 4        = 997 cycles (function call and return)
        delay_1tcy();   // = 998 cycles
        delay_2tcy();   // = 1000 cycles
    }


    // Delay up to 255 milliseconds

    void delay_ms(char ms) {
        char n;
        while(ms--) {    
            n = 62;
            do {
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("goto $+1");
                asm("nop");
                // decrease if not zero - decfsz (1 cycle if !=0, 2 cycles otherwise)
                // goto loop start  - 2 cycles

            } while(--n);
            // = 16 cycles x [n+1] + 1 (from last decfsz)
        }
    }
#endif