
// Delay routines for 16 Mhz clock.
//
// 16 Mhz = 4 Hz per cycle = 4 operations/cycles per uS
//

#ifndef __DELAYH_INIT

    #define __DELAYH_INIT

    /* 16 Mhz, for internal XC8 delay macros ( __delay_ms() for example)
     *
     * Using the same keyword for own delay routines
     */
    #define _XTAL_FREQ 16000000UL

    #define delay_1tcy()    asm("nop")
    #define delay_2tcy()    asm("goto $+1")
    #define delay_4tcy()    asm("goto $+1");asm("goto $+1");
    #define delay_10tcy()   delay_2tcy();delay_4tcy();delay_4tcy();

    /* 16 Mhz , 16m ticks a second, 4m cycles/s, 4000/ms, 4/us */

    #if _XTAL_FREQ == 16000000UL

        #define delay_500ns()   delay_2tcy();
        #define delay_1us()     delay_4tcy();
        #define delay_5us()     delay_10tcy();delay_10tcy();
        #define delay_10us()    delay_5us();delay_5us();

    #endif

    /* 8 Mhz , 8.000.000 ticks a second, 2.000.000 cycles/s, 2000/ms, 2/us */

    #if _XTAL_FREQ == 8000000UL

        #define delay_500ns()   delay_1tcy()
        #define delay_1us()     delay_2tcy()
        #define delay_5us()     delay_10tcy();
        #define delay_10us()    delay_5us();delay_5us();

    #endif

    /* 4 Mhz , 4.000.000 ticks a second, 1.000.000 cycles/s, 1000/ms, 1/us */

    #if _XTAL_FREQ == 4000000UL

        #define delay_500ns()   /* makes no sense, can't do it at 4 Mhz */
        #define delay_1us()     delay_1tcy()
        #define delay_5us()     delay_4tcy();delay_4tcy();delay_1tcy();
        #define delay_10us()    delay_5us();delay_5us();

    #endif

    #define delay_5ms()     delay_ms(5);
    #define delay_10ms()    delay_ms(10);
    #define delay_25ms()    delay_ms(25);
    #define delay_50ms()    delay_ms(50);

#endif
extern void delay_25us();
extern void delay_50us();
extern void delay_100us();

extern void delay_1ms();
extern void delay_ms(char ms);