#include <xc.h>
#include "display.h"
#include "delay.h"


const char digits[20] = {/* pfedcgba p = dot, point */
                    0b01111011,  /* 0 */
                    0b00001010,  /* 1 */
                    0b00110111,  /* 2 */
                    0b00011111,  /* 3 */
                    0b01001100,  /* 4 */
                    0b01011101,  /* 5 */
                    0b01111101,  /* 6 */
                    0b00001011,  /* 7 */
                    0b01111111,  /* 8 */
                    0b01011111,  /* 9 */
                    0b00000100,  /* 10 - (negative sign) */
                    0b01110000,  /* 11 L (for Lo battery) */
                    0b00111100,  /* 12 0 (small O)    */
                    0b01101111,  /* 13  A */
                    0b01111100,  /* 14  b */
                    0b00110100,  /* 15  c */
                    0b00111110,  /* 16  d */
                    0b01110101,  /* 17  E */
                    0b01100101,  /* 18  F */
                    0b00000000,  /* 19  all off */
                   /* pfedcgba */
                };

char display[4] = {DIGIT_OFF,DIGIT_OFF,DIGIT_OFF,DIGIT_OFF};
char decimals[4] = {0,0,0,0};


void display_setdigit(char digit, char value) {
    if (value<20) { /* if we send the code by treating value as number, pass it */
        display[digit] = value;
        return;
    }
    /* the below are in case we use characters as parameter, which have other
     * ASCII codes
     */
    if ((value>64) && (value<71)) { /* A-F  A : 65 in ascii table*/
        display[digit] = value-52; /*-65+13; */
        return;
    }
    if ((value>96) && (value<103)) { /* a-f a: 97 in ascii table */
        display[digit] = value-84; /*-97+13;*/
        return;
    }
    switch (value){

        case '-': display[digit] = 10; break;
        case 'L': display[digit] = 11; break; /* L uppercase */
        case 'l': display[digit] = 11; break;  /* L lowercase */
        case 'O': display[digit] = 12; break; /* O uppercase */
        case 'o': display[digit] = 12; break; /* O lowercase */
        default: display[digit] = 19; // 19 DIGIT_OFF;  /* off */
    }
}

void display_setnumber(unsigned short value, unsigned short decimalcount) {
    unsigned short temp = value;
    char position = 4;
    char remainder = 0;
    display[0] = DIGIT_OFF;
    display[1] = DIGIT_OFF;
    display[2] = DIGIT_OFF;
    display[3] = DIGIT_OFF;
    decimals[1] = 0;
    decimals[2] = 0;
    if (temp>9999) {
        temp = temp % 10000;  /* make sure number is only 4 digits */
    }
    if (temp==0) {
        display[3] = 0; /* if 0, nothing would happen below, so force it */
    }
    if (temp<100) { /* add 0 in case we have a number like 0.15 to display */
        display[1] = 0;
    }
    if (temp < 10) {
        display[2] = 0;
    }
    while (temp > 0) {
        remainder = temp % 10;
        display[position-1] = remainder;
        position = position-1;
        temp = temp / 10;
    }
    /* enable lower dot if there are two decimals after the temperature (<100c) */

    if (decimalcount == 2) {
        decimals[1] = 1;
    }
    /* enable both dots if there's only one decimal ( > 100c) */
    if (decimalcount == 1) {
        decimals[1] = 1;
        decimals[2] = 1;
    }
}

/* For each one of the 4 "characters" in the display[] array, it picks the
 * suitable bits from digits[] array and turns on the proper pins in PORT D.
 * PORT B pins 0-3 are connected to four transistors that in turn connect the
 * cathodes of the display to ground.
 *
 * displaytime = loop through the digits until this time is met or exceeded (ms)
 */

void display_showdigit(char digit) {
    switch (digit) {    /* turn on the transistor dedicated to digit wanted */
        case 0: PORTB = 1; break;
        case 1: PORTB = 2; break;
        case 2: PORTB = 4; break;
        case 3: PORTB = 8; break;
        default: PORTB = 1;
    }
    char j=0;
    if (decimals[digit]==1) { /* is dot supposed to be on? then add it to digit */
        j = 128; //0b10000000;
    }
    PORTD = digits[display[digit]] + j; /* get the bits from digits, + opt. dot*/
    delay_ms(LED_TIME_DIGIT); /* wait so many ms before turning off digit */
    PORTB = 0;
    PORTD = 0;
    //delay_5us(); /* not really neeeded, but may prevent bleeding between digits */
}

void display_show(unsigned short displaytime) {
    unsigned short k;
    k=0;
    /* not ideal, at low frequencies (think 4 Mhz or lower), the call and return
     * from functions could add to flicker caused by multiplexing. But a good
     * compiler should optimize this anyway. 
     */
    while (k<displaytime) {
        display_showdigit(0);
        display_showdigit(1);
        display_showdigit(2);
        display_showdigit(3);
        k=k+LED_TIME_ALL;
    }
    /* TODO: maybe optimize this by not calling showdigit if digit is DIGIT_OFF
     * (must account then for it in the K value (add led_time_digit for every
     * call instead of using led_time_all)
     */
}